#!/bin/bash


source $PWD/lib/vector_calc.sh
# 默认参数
NET_LOG_DIR="/root/net_delay"
CPU_LOG_DIR="/root/cpu_delay"
IO_LOG_DIR="/root/io_delay"
MM_LOG_DIR="/root/mm_delay"
CON_LOG_DIR="/root/con_status"
NET_INTERVAL=60
CPU_INTERVAL=60
IO_INTERVAL=60
MM_INTERVAL=60

# 帮助信息
usage() {
  echo "Usage: $0 [-n NET_LOG_DIR] [-c CPU_LOG_DIR] [-i IO_LOG_DIR] [-m MM_LOG_DIR] [-t NET_INTERVAL] [-u CPU_INTERVAL] [-v IO_INTERVAL] [-w MM_INTERVAL] [--net] [--cpu] [--io] [--mem]"
  exit 1
}

# 初始化监控标志
NET_MONITOR=false
CPU_MONITOR=false
IO_MONITOR=false
MEM_MONITOR=false
CON_MONITOR=false  

# 解析命令行参数
while [[ $# -gt 0 ]]; do
  case "$1" in
    -n) NET_LOG_DIR="$2"; shift 2 ;;
    -c) CPU_LOG_DIR="$2"; shift 2 ;;
    -i) IO_LOG_DIR="$2"; shift 2 ;;
    -m) MM_LOG_DIR="$2"; shift 2 ;;
    -t) NET_INTERVAL="$2"; shift 2 ;;
    -u) CPU_INTERVAL="$2"; shift 2 ;;
    -v) IO_INTERVAL="$2"; shift 2 ;;
    -w) MM_INTERVAL="$2"; shift 2 ;;
    -x) CON_LOG_DIR="$2"; shift 2 ;;  # 新增con日志目录的参数
    -y) CON_INTERVAL="$2"; shift 2 ;;  # 新增con时间间隔的参数
    --net) NET_MONITOR=true; shift ;;
    --cpu) CPU_MONITOR=true; shift ;;
    --io) IO_MONITOR=true; shift ;;
    --mem) MEM_MONITOR=true; shift ;;
    --con) CON_MONITOR=true; shift ;;  # 新增con模块的选项
    *) usage ;;
  esac
done

# 如果未指定任何监控，默认执行所有监控
if [ "$NET_MONITOR" = false ] && [ "$CPU_MONITOR" = false ] && [ "$IO_MONITOR" = false ] && [ "$MEM_MONITOR" = false ]; then
  NET_MONITOR=true
  CPU_MONITOR=true
  IO_MONITOR=true
  MEM_MONITOR=true
  CON_MONITOR=true  # 默认启动con模块
fi

# 检查并创建日志目录
check_and_create_dir() {
  local dir=$1
  if [ ! -d "$dir" ]; then
    echo "日志目录 $dir 不存在，创建中..."
    mkdir -p "$dir"
  fi
}

declare -A cpu_delay #
stage=0

pmm_delay() {
  local pre_info="$1"
  local cur_info="$2"
  local name="$3"
  local delta
  local p_cent

  delt=$(vector_subtract "$cur_info" "$pre_info")
  p_cent=$(calculate_percentage "`echo $delt | awk '{for(i=1; i<=NF-2; i++) printf $i " "; print ""}'`")
  echo $name : $delt >> $MM_LOG_DIR/lat.raw
  echo $name : $p_cent  >> $MM_LOG_DIR/lat.log

}

pcpu_delay() {
  local pre_info="$1"
  local cur_info="$2"
  local name="$3"
  local delta
  local p_cent

  delt=$(vector_subtract "$cur_info" "$pre_info")
  p_cent=$(calculate_percentage "`echo $delt | awk '{for(i=1; i<=NF-2; i++) printf $i " "; print ""}'`")
  echo $name : $delt >> $CPU_LOG_DIR/lat.raw
  echo $name : $p_cent  >> $CPU_LOG_DIR/lat.log

}


# 网络延迟监控
monitor_net_delay() {
  check_and_create_dir "$NET_LOG_DIR"
  while true
  do
    timestamp=$(date)
    echo "当前时间戳: $timestamp" >> $NET_LOG_DIR/system.log 2>&1
    /usr/share/bili_tools/net/tcp_conn_top -n 1 -o rtt_avg >> $NET_LOG_DIR/system.log 2>&1
    sleep $NET_INTERVAL
  done
}

# CPU延迟监控
monitor_cpu_delay() {

check_and_create_dir "$CPU_LOG_DIR"
#mkdir /root/sched_lat > /dev/null 2>&1
#rm -fr /root/sched_lat/*

while true
do
  date +"%Y-%m-%d %H:%M:%S" | tee -a $CPU_LOG_DIR/lat.log $CPU_LOG_DIR/lat.raw
  cpu_delay["cur_burst_wait"]=`cat /sys/fs/cgroup/cpu/kubepods/burstable/cpuacct.bili_stat | sed -n '5p'`
  cpu_delay["cur_k8s_wait"]=`cat /sys/fs/cgroup/cpu/kubepods/cpuacct.bili_stat | sed -n '5p'`
  cpu_delay["cur_k8s_pwait"]=`cat /sys/fs/cgroup/cpu/kubepods/cpuacct.bili_stat | sed -n '6p'`
  if [ $stage -eq 1 ];then
    pcpu_delay "${cpu_delay["pre_burst_wait"]}" "${cpu_delay["cur_burst_wait"]}" "burst wait"
    pcpu_delay "${cpu_delay["pre_k8s_wait"]}" "${cpu_delay["cur_k8s_wait"]}" "k8s wait"
    pcpu_delay "${cpu_delay["pre_k8s_pwait"]}" "${cpu_delay["cur_k8s_pwait"]}" "k8s pwait"
  fi
  cpu_delay["pre_burst_wait"]=${cpu_delay["cur_burst_wait"]}
  cpu_delay["pre_k8s_wait"]=${cpu_delay["cur_k8s_wait"]}
  cpu_delay["pre_k8s_pwait"]=${cpu_delay["cur_k8s_pwait"]}

  CON_PODS=$(crictl ps  | grep -vE 'colocation-device|cadvisor|coredump-moni|kruise-daemon|csi-xfs-agent|node-problem-detector|probe-checknode' | grep 'main'|awk '{print $1"_"$NF}')
  for i in $CON_PODS
  do
    ctn_name=`echo $i | cut -d'_' -f2`
    ctn_id=`echo $i | cut -d'_' -f1`
    ctn_path=`find /sys/fs/cgroup/cpu/ -name *${ctn_id}* | grep -v mount`
    pod_path=`dirname $ctn_path`
    pod_name=`dirname $ctn_path | awk -F'/' '{print$NF}'`
    pod_type=`dirname $pod_path | awk -F'/' '{print$NF}'`
    cpu_delay["cur_${ctn_name}_wait"]=`cat $ctn_path/cpuacct.bili_stat | sed -n '5p'`
    cpu_delay["cur_${pod_name}_wait"]=`cat $pod_path/cpuacct.bili_stat | sed -n '5p'`

    if [ $stage -eq 1 ];then
      pcpu_delay "${cpu_delay["pre_${ctn_name}_wait"]}" "${cpu_delay["cur_${ctn_name}_wait"]}" "$ctn_name"
      pcpu_delay "${cpu_delay["pre_${pod_name}_wait"]}" "${cpu_delay["cur_${pod_name}_wait"]}" "$pod_name"
    fi

    cpu_delay["pre_${ctn_name}_wait"]=${cpu_delay["cur_${ctn_name}_wait"]}
    cpu_delay["pre_${pod_name}_wait"]=${cpu_delay["cur_${pod_name}_wait"]}
  done
  stage=1
  sleep $CPU_INTERVAL
  echo ------ | tee -a $CPU_LOG_DIR/lat.log $CPU_LOG_DIR/lat.raw
done

}

# I/O延迟监控
monitor_io_delay() {
  check_and_create_dir "$IO_LOG_DIR"
  while true
  do
    timestamp=$(date)
    echo "当前时间戳: $timestamp" >> $IO_LOG_DIR/system.log 2>&1
    iostat -xm >> $IO_LOG_DIR/system.log 2>&1
    sleep $IO_INTERVAL
  done
}

# 内存延迟监控
monitor_mem_delay() {
check_and_create_dir "$MM_LOG_DIR"
while true
do
  date +"%Y-%m-%d %H:%M:%S" | tee -a $MM_LOG_DIR/lat.log $MM_LOG_DIR/lat.raw
  mm_delay["cur_burst_reclaim"]=`cat /sys/fs/cgroup/memory/kubepods/burstable/memory.bili_stat | sed -n '2p'`
  mm_delay["cur_k8s_reclaim"]=`cat /sys/fs/cgroup/memory/kubepods/memory.bili_stat | sed -n '2p'`
  if [ $stage -eq 1 ];then
    pmm_delay "${mm_delay["pre_burst_reclaim"]}" "${mm_delay["cur_burst_reclaim"]}" "burst reclaim"
    pmm_delay "${mm_delay["pre_k8s_reclaim"]}" "${mm_delay["cur_k8s_reclaim"]}" "k8s reclaim"
  fi
  mm_delay["pre_burst_reclaim"]=${mm_delay["cur_burst_reclaim"]}
  mm_delay["pre_k8s_reclaim"]=${mm_delay["cur_k8s_reclaim"]}

  CON_PODS=$(crictl ps  | grep -vE 'colocation-device|cadvisor|coredump-moni|kruise-daemon|csi-xfs-agent|node-problem-detector|probe-checknode' | grep 'main'|awk '{print $1"_"$NF}')
  for i in $CON_PODS
  do
    ctn_name=`echo $i | cut -d'_' -f2`
    ctn_id=`echo $i | cut -d'_' -f1`
    ctn_path=`find /sys/fs/cgroup/memory/ -name *${ctn_id}* | grep -v mount`
    pod_path=`dirname $ctn_path`
    pod_name=`dirname $ctn_path | awk -F'/' '{print$NF}'`
    pod_type=`dirname $pod_path | awk -F'/' '{print$NF}'`
    mm_delay["cur_${ctn_name}_reclaim"]=`cat $ctn_path/memory.bili_stat | sed -n '2p'`
    mm_delay["cur_${pod_name}_reclaim"]=`cat $pod_path/memory.bili_stat | sed -n '2p'`

    if [ $stage -eq 1 ];then
      pmm_delay "${mm_delay["pre_${ctn_name}_reclaim"]}" "${mm_delay["cur_${ctn_name}_reclaim"]}" "$ctn_name"
      pmm_delay "${mm_delay["pre_${pod_name}_reclaim"]}" "${mm_delay["cur_${pod_name}_reclaim"]}" "$pod_name"
    fi

    mm_delay["pre_${ctn_name}_reclaim"]=${mm_delay["cur_${ctn_name}_reclaim"]}
    mm_delay["pre_${pod_name}_reclaim"]=${mm_delay["cur_${pod_name}_reclaim"]}
  done
  stage=1
  sleep $MM_INTERVAL
  echo ------ | tee -a $MM_LOG_DIR/lat.log $MM_LOG_DIR/lat.raw
done
}

monitor_con_delay() {
  check_and_create_dir "$CON_LOG_DIR"
  while true
  do
    timestamp=$(date)
    echo "当前时间戳: $timestamp" >> $CON_LOG_DIR/system.log 2>&1
    /usr/share/bili_tools/lsctn.sh -s >> $CON_LOG_DIR/system.log 2>&1
    sleep $CON_INTERVAL
  done
}

# 根据标志并行执行监控任务
if [ "$NET_MONITOR" = true ]; then
  monitor_net_delay &
fi

if [ "$CPU_MONITOR" = true ]; then
  monitor_cpu_delay &
fi

if [ "$IO_MONITOR" = true ]; then
  monitor_io_delay &
fi

if [ "$MEM_MONITOR" = true ]; then
  monitor_mem_delay &
fi

if [ "$CON_MONITOR" = true ]; then
  monitor_con_delay &
fi

# 等待所有后台任务完成
wait

