#!/bin/bash

HELP="
\t-f -- filter container by info
\t-p -- show PID related container
\t-s -- show setup & statistics
"

function phelp()
{
	echo "Usage: `basename $0` [-p PID] [-s] [-f FILTER]"
	printf "$HELP"
	exit
}

NR_CPUS=`lscpu | grep "^CPU(s):" | awk '{print $2}'`
SHOW_DETAIL=0
while getopts ":f:p:s" o; do
	case "${o}" in
		f)
			FILTER=${OPTARG}
			;;
		p)
			TPID=${OPTARG}
			TCID=`cat /proc/$TPID/cgroup | grep cpuacct`
			TID=`basename $TCID | cut -c1-6`
			;;
		s)
			SHOW_DETAIL=1
			;;
		*)
			phelp
			;;
	esac
done

if [ "$TPID" != "" ] && [ "$TID" == "" ];then
	echo "No container own $TPID"
	phelp
fi

function pbilihead()
{
	printf "%-10s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t\n" "LAT(ms)" "<4" ">4" ">16" ">64" ">256" ">1024" ">4096" "avg"
}

function pbilistat()
{
	DATA=$1
	echo $DATA | awk '{P1=($1+$2)*100/$16} {P2=($3+$4)*100/$16} {P3=($5+$6)*100/$16} {P4=($7+$8)*100/$16} {P5=($9+$10)*100/$16} {P6=($11+$12)*100/$16} {P7=($13+$14)*100/$16} {AVG=$15/$16} END{printf("%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f\n",P1, P2, P3, P4, P5, P6, P7, AVG)}'
}

function pcpuacct()
{
	CGROUP=$1
	echo "cpuacct.bili_stat" = 

	NRL=0
	cat $CGROUP/cpuacct.bili_stat | while read LINE;do
		NRL=$(($NRL + 1))
		if [ $NRL -eq 1 ];then
			printf "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t\n" "usage" "us" "ni" "sy" "id" "wa" "hi" "si" "st" "gu"
			echo $LINE | awk -vnr_cpus="$NR_CPUS" '{ALL=$1+$2+$3+$4+$5+$6+$7+$8+$9} {P1=$1*100/ALL} {P2=$2*100/ALL} {P3=$3*100/ALL} {P4=$4*100/ALL} {P5=$5*100/ALL} {P6=$6*100/ALL} {P7=$7*100/ALL} {P8=$8*100/ALL} {P9=$9*100/ALL} {USAGE=(100-P4)*nr_cpus} END{printf("%0.2f\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\t%0.2f%%\n", USAGE, P1, P2, P3, P4, P5, P6, P7, P8, P9)}'
			continue
		elif [ $NRL -eq 5 ];then
			pbilihead
			printf "%-10s\t" "WAIT:"
		elif [ $NRL -eq 6 ];then
			printf "%-10s\t" "PSE_WAIT:"
		elif [ $NRL -eq 7 ];then
			printf "%-10s\t" "BLOCK:"
		elif [ $NRL -eq 8 ];then
			printf "%-10s\t" "IOBLOCK:"
		else
			continue
		fi
		pbilistat "$LINE"
	done
}

function pmemory()
{
	CGROUP=$1
	echo "memory.bili_stat" = 

	NRL=0
	cat $CGROUP/memory.bili_stat | while read LINE;do
		NRL=$(($NRL + 1))
		if [ $NRL -gt 3 ];then
			continue
		elif [ $NRL -eq 1 ];then
			pbilihead
			printf "%-10s\t" "GLOBAL:"
		elif [ $NRL -eq 2 ];then
			printf "%-10s\t" "CGROUP:"
		elif [ $NRL -eq 3 ];then
			printf "%-10s\t" "COMPACT:"
		fi
		pbilistat "$LINE"
	done

	echo "memory.numa_stat" =
	cat $CGROUP/memory.numa_stat | head -n4
	LIMIT=`cat $CGROUP/memory.limit_in_bytes`
	echo "memory.limit_in_bytes" = $LIMIT
	USAGE=`cat $CGROUP/memory.usage_in_bytes`
	if [ $LIMIT -gt 1 ];then
		PCT=`echo "$USAGE*100/$LIMIT" | bc -l`
		printf "memory.usage_in_bytes=%llu(%0.2f%%)\n" $USAGE $PCT
	else
		echo "memory.usage_in_bytes" = $USAGE
	fi
}

function pcpuset()
{
	CGROUP=$1
	CPUS=`cat $CGROUP/cpuset.cpus`
	echo "cpuset.cpus" = $CPUS
	./lsaffine.sh "$CPUS"
	echo "cpuset.mems" = `cat $CGROUP/cpuset.mems`
}

function pcpu()
{
	CGROUP=$1
	echo "cpu.cfs_quota_us" = `cat $CGROUP/cpu.cfs_quota_us`
	echo "cpu.cfs_period_us" = `cat $CGROUP/cpu.cfs_period_us`
	echo "cpu.stat" = 
	cat $CGROUP/cpu.stat
}

./lstopo.sh > /dev/null
ls -d /sys/fs/cgroup/cpuacct/kubepods/burstable/*/ | while read POD;do
	ls -d $POD/*/ | while read CTN;do
		ID=`basename $CTN | cut -c1-6`
		if [ "$TID" != "" ] && [ "$ID" != "$TID" ];then
			continue
		fi
		INFO=`crictl ps | awk '{print $1 " " $10 " " $6 " " $7}' | grep $ID`
		if [ "$INFO" == "" ];then
			continue
		fi
		if [ "$FILTER" != "" ];then
			if [ "`echo $INFO | grep $FILTER`" == "" ];then
				continue
			fi
		fi
		CG=`echo $CTN | sed 's/\/sys\/fs\/cgroup\/cpuacct\///'`
		echo $CG = `echo $INFO | awk '{print $2 ":" $4}'`
		if [ $SHOW_DETAIL -eq 0 ];then
			continue
		fi

        pcpuacct "/sys/fs/cgroup/cpuacct/$CG"
		pmemory "/sys/fs/cgroup/memory/$CG"
		pcpuset "/sys/fs/cgroup/cpuset/$CG"
		pcpu "/sys/fs/cgroup/cpu/$CG"
	done
done
